import { Component, OnInit } from '@angular/core';
import { MatSnackBar } from '@angular/material/snack-bar';
import { AuthService } from '../../services/auth.service';
import { PasswordService } from '../../services/password.service';
import { CommonModule } from '@angular/common';
import { FormsModule } from '@angular/forms';
import { MatCardModule } from '@angular/material/card';
import { MatFormFieldModule } from '@angular/material/form-field';
import { MatInputModule } from '@angular/material/input';
import { MatButtonModule } from '@angular/material/button';
import { MatIconModule } from '@angular/material/icon';
import { MatSliderModule } from '@angular/material/slider';
import { MatCheckboxModule } from '@angular/material/checkbox';
import { MatTooltipModule } from '@angular/material/tooltip';
import { MatSnackBarModule } from '@angular/material/snack-bar';
import { Router } from '@angular/router';
import { MatDialogModule, MatDialog, MatDialogRef } from '@angular/material/dialog';
import { ThemeService } from '../../services/theme.service';

@Component({
  selector: 'app-save-password-dialog',
  template: `
    <h2 mat-dialog-title>Save Password</h2>
    <mat-dialog-content>
      <mat-form-field class="full-width">
        <mat-label>Description</mat-label>
        <input matInput 
               [(ngModel)]="description" 
               placeholder="e.g., Gmail Account"
               (keyup.enter)="savePassword()"
               #descInput="ngModel"
               required
               minlength="3"
               maxlength="100">
        <mat-error *ngIf="descInput.errors?.['required']">
          Description is required
        </mat-error>
        <mat-error *ngIf="descInput.errors?.['minlength']">
          Description must be at least 3 characters
        </mat-error>
        <mat-error *ngIf="descInput.errors?.['maxlength']">
          Description cannot exceed 100 characters
        </mat-error>
      </mat-form-field>
    </mat-dialog-content>
    <mat-dialog-actions align="end">
      <button mat-button mat-dialog-close>Cancel</button>
      <button mat-raised-button 
              color="primary" 
              [disabled]="!description.trim() || description.length < 3"
              (click)="savePassword()">
        Save
      </button>
    </mat-dialog-actions>
  `,
  styles: [`
    .full-width { 
      width: 100%; 
      margin-bottom: 15px;
    }
    mat-form-field {
      margin-top: 10px;
    }
  `],
  standalone: true,
  imports: [
    CommonModule,
    FormsModule,
    MatDialogModule,
    MatButtonModule,
    MatFormFieldModule,
    MatInputModule
  ]
})
export class SavePasswordDialogComponent {
  description = '';

  constructor(public dialogRef: MatDialogRef<SavePasswordDialogComponent>) {}

  savePassword(): void {
    const trimmedDesc = this.description.trim();
    if (trimmedDesc && trimmedDesc.length >= 3) {
      this.dialogRef.close(trimmedDesc);
    }
  }
}

@Component({
  selector: 'app-password-generator',
  templateUrl: './password-generator.component.html',
  styleUrls: ['./password-generator.component.scss'],
  standalone: true,
  imports: [
    CommonModule,
    FormsModule,
    MatCardModule,
    MatFormFieldModule,
    MatInputModule,
    MatButtonModule,
    MatIconModule,
    MatSliderModule,
    MatCheckboxModule,
    MatTooltipModule,
    MatSnackBarModule,
    MatDialogModule
  ]
})
export class PasswordGeneratorComponent implements OnInit {
  generatedPassword = '';
  passwordLength = 16;
  includeUppercase = true;
  includeLowercase = true;
  includeNumbers = true;
  includeSpecial = true;
  isLoggedIn = false;
  isDarkTheme$;

  constructor(
    private snackBar: MatSnackBar,
    private authService: AuthService,
    private passwordService: PasswordService,
    private router: Router,
    private dialog: MatDialog,
    private themeService: ThemeService
  ) {
    this.isDarkTheme$ = this.themeService.isDarkTheme$;
  }

  ngOnInit(): void {
    this.generatePassword();
    this.authService.isAuthenticated$.subscribe(
      isAuthenticated => this.isLoggedIn = isAuthenticated
    );
  }

  toggleTheme(): void {
    this.themeService.toggleTheme();
  }

  generatePassword(): void {
    const charset: { [key: string]: string } = {
      uppercase: 'ABCDEFGHIJKLMNOPQRSTUVWXYZ',
      lowercase: 'abcdefghijklmnopqrstuvwxyz',
      numbers: '0123456789',
      special: '!@#$%^&*()_+-=[]{}|;:,.<>?'
    };

    let availableChars = '';
    if (this.includeUppercase) availableChars += charset['uppercase'];
    if (this.includeLowercase) availableChars += charset['lowercase'];
    if (this.includeNumbers) availableChars += charset['numbers'];
    if (this.includeSpecial) availableChars += charset['special'];

    if (availableChars === '') {
      this.snackBar.open('Please select at least one character type', 'Close', { duration: 3000 });
      return;
    }

    let password = '';
    for (let i = 0; i < this.passwordLength; i++) {
      const randomIndex = Math.floor(Math.random() * availableChars.length);
      password += availableChars[randomIndex];
    }

    this.generatedPassword = password;
  }

  onLengthChange(value: number | null): void {
    if (value !== null) {
      this.passwordLength = value;
      this.generatePassword();
    }
  }

  copyToClipboard(): void {
    navigator.clipboard.writeText(this.generatedPassword).then(() => {
      this.snackBar.open('Password copied to clipboard!', 'Close', { duration: 2000 });
    });
  }

  savePassword(): void {
    if (!this.isLoggedIn) {
      this.snackBar.open('Please log in to save passwords', 'Close', { duration: 3000 });
      return;
    }

    if (!this.generatedPassword) {
      this.snackBar.open('Please generate a password first', 'Close', { duration: 3000 });
      return;
    }

    const dialogRef = this.dialog.open(SavePasswordDialogComponent, {
      width: '400px',
      disableClose: false
    });

    dialogRef.afterClosed().subscribe(description => {
      if (description) {
        this.passwordService.savePassword({
          description: description.trim(),
          password: this.generatedPassword
        }).subscribe({
          next: (response) => {
            this.snackBar.open('Password saved successfully!', 'Close', { duration: 2000 });
            this.router.navigate(['/passwords']);
          },
          error: (error) => {
            let errorMessage = 'Failed to save password';
            
            if (error.message) {
              errorMessage = error.message;
            }
            
            this.snackBar.open(errorMessage, 'Close', {
              duration: 5000,
              panelClass: ['error-snackbar']
            });
          }
        });
      }
    });
  }
}
