import { Component, OnInit, ViewChild, TemplateRef } from '@angular/core';
import { CommonModule } from '@angular/common';
import { FormsModule } from '@angular/forms';
import { RouterModule } from '@angular/router';
import { MatCardModule } from '@angular/material/card';
import { MatListModule } from '@angular/material/list';
import { MatButtonModule } from '@angular/material/button';
import { MatIconModule } from '@angular/material/icon';
import { MatInputModule } from '@angular/material/input';
import { MatFormFieldModule } from '@angular/material/form-field';
import { MatProgressSpinnerModule } from '@angular/material/progress-spinner';
import { MatSnackBar } from '@angular/material/snack-bar';
import { MatDialog, MatDialogModule } from '@angular/material/dialog';
import { MatTooltipModule } from '@angular/material/tooltip';
import { MatDividerModule } from '@angular/material/divider';
import { trigger, transition, style, animate } from '@angular/animations';
import { ThemeService } from '../../services/theme.service';

import { PasswordService, SavedPassword } from '../../services/password.service';

interface PasswordWithVisibility extends SavedPassword {
  showPassword: boolean;
}

@Component({
  selector: 'app-password-list',
  templateUrl: './password-list.component.html',
  styleUrls: ['./password-list.component.scss'],
  standalone: true,
  imports: [
    CommonModule,
    FormsModule,
    RouterModule,
    MatCardModule,
    MatListModule,
    MatButtonModule,
    MatIconModule,
    MatInputModule,
    MatFormFieldModule,
    MatProgressSpinnerModule,
    MatDialogModule,
    MatTooltipModule,
    MatDividerModule
  ],
  animations: [
    trigger('fadeInOut', [
      transition(':enter', [
        style({ opacity: 0, transform: 'translateY(10px)' }),
        animate('300ms ease-out', style({ opacity: 1, transform: 'translateY(0)' }))
      ]),
      transition(':leave', [
        animate('300ms ease-in', style({ opacity: 0, transform: 'translateY(10px)' }))
      ])
    ])
  ]
})
export class PasswordListComponent implements OnInit {
  @ViewChild('deleteDialog') deleteDialog!: TemplateRef<any>;
  
  passwords: PasswordWithVisibility[] = [];
  filteredPasswords: PasswordWithVisibility[] = [];
  isLoading = true;
  searchTerm = '';
  isDarkTheme$;

  constructor(
    private passwordService: PasswordService,
    private snackBar: MatSnackBar,
    private dialog: MatDialog,
    private themeService: ThemeService
  ) {
    this.isDarkTheme$ = this.themeService.isDarkTheme$;
  }

  ngOnInit(): void {
    this.loadPasswords();
  }

  loadPasswords(): void {
    this.isLoading = true;
    this.passwordService.getPasswords().subscribe({
      next: (passwords) => {
        this.passwords = passwords.map(p => ({ ...p, showPassword: false }));
        this.filteredPasswords = [...this.passwords];
        this.isLoading = false;
      },
      error: (error) => {
        this.isLoading = false;
        this.snackBar.open('Error loading passwords', 'Close', { 
          duration: 3000,
          panelClass: ['error-snackbar']
        });
        console.error('Error loading passwords:', error);
      }
    });
  }

  filterPasswords(): void {
    if (!this.searchTerm) {
      this.filteredPasswords = [...this.passwords];
      return;
    }

    const searchTerm = this.searchTerm.toLowerCase();
    this.filteredPasswords = this.passwords.filter(password =>
      password.description.toLowerCase().includes(searchTerm)
    );
  }

  clearSearch(): void {
    this.searchTerm = '';
    this.filterPasswords();
  }

  togglePasswordVisibility(password: PasswordWithVisibility): void {
    password.showPassword = !password.showPassword;
  }

  copyPassword(password: PasswordWithVisibility): void {
    navigator.clipboard.writeText(password.password);
    this.snackBar.open('Password copied to clipboard!', 'Close', { 
      duration: 2000,
      panelClass: ['success-snackbar']
    });
  }

  deletePassword(password: PasswordWithVisibility): void {
    const dialogRef = this.dialog.open(this.deleteDialog, {
      width: '400px',
      panelClass: 'confirm-dialog'
    });

    dialogRef.afterClosed().subscribe(result => {
      if (result) {
        this.passwordService.deletePassword(password._id).subscribe({
          next: () => {
            this.passwords = this.passwords.filter(p => p._id !== password._id);
            this.filteredPasswords = this.filteredPasswords.filter(p => p._id !== password._id);
            this.snackBar.open('Password deleted successfully', 'Close', { 
              duration: 3000,
              panelClass: ['success-snackbar']
            });
          },
          error: (error) => {
            this.snackBar.open('Error deleting password', 'Close', { 
              duration: 3000,
              panelClass: ['error-snackbar']
            });
            console.error('Error deleting password:', error);
          }
        });
      }
    });
  }

  toggleTheme(): void {
    this.themeService.toggleTheme();
  }
}
